/*Copyright (c) 2003 The Regents of the University of California.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
     This product includes software developed by the University of
     California, Los Angeles and its contributors.
4. Neither the name of the University nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE. */

import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;
import java.io.*;

/* Class that draws flow of packets as a graph representation */
class TrackPanel extends JPanel implements Runnable 
{
    boolean changed;
    Thread relaxer;
    int curtime;
    Node node;
    Vector upvalues;
    Vector lowvalues;
    Vector ratelim;
    int pos;
    int start;
    int mins;
    int secs; 
    int start_time;  
    int oldmax;
    int offset;
    String uplabel,lowlabel;
    Dimension old;
    boolean isVisible;
    String ylab;   
    boolean active;
    Point showPoint;
    boolean paused;

    public TrackPanel(Node n) {
	node = n;
	start_time = -1;
	curtime = 0; 
	setBackground(Color.white);
	changed = false;
	pos = 0;
	upvalues = new Vector(10000);
	lowvalues = new Vector(10000);
	ratelim = new Vector(10000);
	uplabel = new String("Sent BW");
	lowlabel = new String("Received BW");
	old = getSize();
	oldmax = 0;
	offset = -1;
	isVisible = false;
	ylab = new String("");
	active = true;
	paused = false;
    } 
    
    public void handleRateMessage
	(int time, String node, double rlim, int sent, int rec)
    {
	if (start_time == -1)
	    {
		start_time = time;
	    }
	if (time-start_time>ratelim.size()-1)
	    {
		ratelim.setSize(time-start_time);
		upvalues.setSize(time-start_time);
		lowvalues.setSize(time-start_time);
	    }
	
	if (ratelim.size()<=(time-start_time) 
	    || ratelim.get(time-start_time) == null)
	    {
		if (rlim>0)
		    ratelim.add(time-start_time,new Integer((int)rlim));
		
		upvalues.add(time-start_time,new Integer(sent));
		lowvalues.add(time-start_time,new Integer(rec));
	    }
	/* System.out.println(upvalues.toString());
	   System.out.println(lowvalues.toString());
	   System.out.println(ratelim.toString());
	*/
    }
    
    public void showValues(int x, int y)
    {
	showPoint = new Point(x,y);
    }
    
    public void hideValues()
    {
	showPoint = null; 
    }
    
    public void handleFlowMessage
	(int time, String source, int Ninterface, int sent, int rec)
    {
	if (start_time == -1)
	    {
		start_time = time;
	    }
	if (time-start_time>upvalues.size()-1)
	    {
		upvalues.setSize(time-start_time);
		lowvalues.setSize(time-start_time);
	    }
	if(upvalues.size()<=(time-start_time) 
	   || upvalues.get(time-start_time) == null)
	    {
		upvalues.add(time-start_time,new Integer(sent));
		lowvalues.add(time-start_time,new Integer(rec));
	    }
	/*
	  System.out.println(upvalues.toString());
	  System.out.println(lowvalues.toString());
	*/
    }
    
    public void run() {
	while (true) {
	    curtime++;
	    if (isVisible)
		{
		    repaint(new Rectangle(getSize()));		
		    changed = false;
		}
	    try{
		Thread.sleep(1000);
	    }
	    catch(Exception e)
		{}
	}
    }
    
    /* Function that paints the graph */
    public void repaint(Rectangle r) {	
	if (paused)
	    return;
	Graphics g = getGraphics();
	Dimension d = getSize();
	int max = 0;
	/* Number of points on x axis */
	int num = (d.width-30)/10;
	int ly1=-1, ly2=-1, ly3=-1,ly4=-1, ly5=-1;
	int lx1=-1, lx2=-1, lx3=-1,lx4=-1, lx5=-1;
	
	/* find the maixmum value - all y axis will be scaled to that */
    for (int i=0;i<num;i++)    
	{
	    if (upvalues.size()>i 
		&& upvalues.get(upvalues.size()-i-1)!=null
		&& ((Integer)(upvalues.get(upvalues.size()-i-1)))
		.intValue() > max)
		max = ((Integer)upvalues.get(upvalues.size()-i-1)).intValue();
	    if (lowvalues.size()>i 
		&& lowvalues.get(lowvalues.size()-i-1)!=null 
		&&  ((Integer)lowvalues.get(lowvalues.size()-i-1))
		.intValue() > max)
		max = ((Integer)lowvalues.get(lowvalues.size()-i-1)).intValue();
	    if (ratelim.size()>i 
		&& ratelim.get(ratelim.size()-i-1)!=null 
		&&  ((Integer)ratelim.get(ratelim.size()-i-1))
		.intValue() > max)
		max = ((Integer)ratelim.get(ratelim.size()-i-1)).intValue();
	}

    if (max == 0)
	max = 10;
    int dd = 2;
    int k=1000000;
    while(k>10)
	{
	    if (k>=max && k/dd<max)
		break;
	    k =  k/dd;
	    if (dd==2)
		dd=5;
	    else
		dd=2;
	}
    max = k;
    /* Reserve the current slot in arrays */
    if (start_time>0 && curtime-start_time>upvalues.size()-1)
	{
	    upvalues.setSize(curtime-start_time);
	    lowvalues.setSize(curtime-start_time);
	    ratelim.setSize(curtime-start_time);
	}
    
    if (g!=null)
	{
	    /* Draw background, axis and labels */
	    g.setColor(Color.white);
	    g.fillRect(30,d.height/2-10,d.width-30,10);
	    g.fillRect(30,d.height-10,d.width-30,10);
	    g.fillRect(d.width-100,0,100,30);
	    if (old != null && d.width != old.width && d.height != old.height)
		g.fillRect(0,0,d.width, d.height);
	    else
		{
		    g.fillRect(30,20,d.width-30, d.height/2-30);
		    g.fillRect(30,d.height/2+20,d.width-30, d.height/2-30);
		    if (max != oldmax)
			g.fillRect(0,0,30,d.height);
		}
	    oldmax = max;
	    g.setColor(Color.black);
	    old = d;	 
	    g.drawLine(30,d.height/2-10,d.width-10,d.height/2-10);
	    g.drawLine(30,20,30,d.height/2-10);
	    g.drawLine(30,d.height-10,d.width-10,d.height-10);
	    g.drawLine(30,d.height/2+20,30,d.height-10);
	    g.drawString("sec",d.width-30,d.height/2-15);
	    g.drawString("sec",d.width-30,d.height-15);
	    FontMetrics fmt = g.getFontMetrics();
	    int w = fmt.stringWidth(uplabel)+2;
	    g.drawString(uplabel,(d.width-20)/2-w/2,17);
	    w = fmt.stringWidth(lowlabel)+2;
	    g.drawString(lowlabel,(d.width-20)/2-w/2,d.height/2+17);
	    
	    double scale=10;
	    int size = d.height/2-30;
	    String ss = new String();
	    String rs = new String();
	    String rls = new String();
	    for (int i=0;i<num;i++)    
		{
		    /* Draw x axis labels */
		    Font fm = new Font("SansSerif", Font.PLAIN, 10);
		    g.setFont(fm);
		    g.drawLine(30+i*10,d.height/2-8, 30+i*10, d.height/2-12);
		    g.drawLine(30+i*10,d.height-8, 30+i*10, d.height-12);
		    if ((curtime-num+i)%10 == 0 && (curtime-num+i)>=0)
			{
			    int m = (curtime-num+i)/60;
			    String t;
			    if (m!=0)
				if ((curtime-num+i)%60 != 0)
				    t = new String
					(""+m+":"+((curtime-num+i)%60));
				else
				    t = new String(""+m+":00");
			    else 
				t = new String(""+(curtime-num+i));
			    g.setColor(Color.black);
			    g.drawString(t,30+i*10,d.height/2);
			    g.drawString(t,30+i*10,d.height);
			    g.setColor(Color.red);
			}
		    g.setColor(Color.red);
		    int y1, y2, index;
		    /* Draw lines */
		    if (upvalues.size() >= (num-i))
			{
			    index = upvalues.size()-(num-i);
			    if (upvalues.get(index)!=null)
				{
				    y1 = (int)
					(size*((Integer)upvalues.get(index))
					 .intValue()/max);
				    if (y1>size)
					y1 = size;
				    if (ly1 == -1)
					{
					    ly1 = y1;
					    lx1 = 30+i*10; 
					}
				    g.drawLine(lx1,d.height/2-10-ly1,
					       30+i*10,d.height/2-10-y1);
				    g.drawLine(lx1,d.height/2-11-ly1,
					       30+i*10,d.height/2-11-y1);
				    if (showPoint != null)
					{
					    if(showPoint.getX()>=30+i*10 
					       && showPoint.getX()<40+i*10)
						{
						    ss = new String
							(""+((Integer)upvalues
							     .get(index))
							 .intValue()); 
						}
					}
				    ly1 = y1;
				    lx1 = 30+i*10;
				}
			    else
				ly1 = -1;
			}
		    if (lowvalues.size() >= (num-i))
			{
			    index = lowvalues.size()-(num-i);
			    if (lowvalues.get(index)!=null)
				{
				    y1 = (int)
					(size*((Integer)lowvalues.get(index))
					 .intValue()/max);
				    if (y1>size)
					y1 = size;
				    if (ly2 == -1)
					{
					    ly2 = y1;
					    lx2 = 30+i*10; 
					}
				    g.drawLine(lx2,d.height-10-ly2,
					       30+i*10,d.height-10-y1);
				    g.drawLine(lx2,d.height-11-ly2,
					       30+i*10,d.height-11-y1);
				    if (showPoint != null)
					{
					    if(showPoint.getX()>=30+i*10 
					       && showPoint.getX()<40+i*10)
						{
						    rs = new String
							(""+((Integer)lowvalues
							     .get(index))
							 .intValue()); 
						}
					}
				    ly2 = y1;
				    lx2 = 30+i*10;
				}
			    else
				ly2 = -1;
			}
		    g.setColor(Color.blue);
		    if (ratelim.size() >= (num-i))
			{
			    index = ratelim.size()-(num-i);
			    if (ratelim.get(index)!=null)
				{
				    y1 = (int)(size
					       *((Integer)ratelim.get(index))
					       .intValue()/max);
				    if (y1>size)
					y1 = size;
				    if (ly3 == -1)
					{
					    ly3 = y1;
					    lx3 = 30+i*10;
					}
				    g.drawLine(lx3,d.height/2-10-ly3,
					       30+i*10,d.height/2-10-y1);
				    g.drawLine(lx3,d.height/2-11-ly3,
					       30+i*10,d.height/2-11-y1);
				    if (showPoint != null)
					{
					    if(showPoint.getX()>=30+i*10 
					       && showPoint.getX()<40+i*10)
						{
						    rls = new String
							(""+((Integer)ratelim
							     .get(index))
							 .intValue()); 
						}
					}
				    ly3 = y1;
				    lx3 = 30+i*10;
				}
			    else
				ly3 = -1;
			}
		    g.setColor(Color.black);
		}
	    int div;
	    if (max>=1000)
		div = 1000;
	    else
		div = 1;
	    Font ftm = new Font("SansSerif", Font.PLAIN, 12);
	    g.setFont(ftm);
	    
	    if(div == 1000)
		{
		    if (!ylab.equals("KBps"))
			{
			    if(!ylab.equals(""))
				{
				    g.setColor(Color.white);
				    g.drawString(ylab,40,18);
				    g.drawString(ylab,40,d.height/2+18);
				}
			    ylab = new String("KBps");
			}	
		}
	    else
		{
		    if (!ylab.equals("Bps"))
			{
			    if(!ylab.equals(""))
				{
				    g.setColor(Color.white);
				    g.drawString(ylab,40,18);
				    g.drawString(ylab,40,d.height/2+18);
				}
			    ylab = new String("Bps");
			}	
		}
	    g.setColor(Color.black);
	    g.drawString(ylab,40,18);
	    g.drawString(ylab,40,d.height/2+18);
	    
	    Font fm = new Font("SansSerif", Font.PLAIN, 10);
	    g.setFont(fm);
	    
	    double pos = d.height/2-10;
	    String s;
	    int offset;
	    if (max/div < 10)
		offset = fmt.stringWidth(" 5"+max/div);
	    else
		offset = fmt.stringWidth(""+max/div);
	    for(int j=0;j<=max;j+=max/scale)
		{
		    if (max/div < 10)
			{
			    s = new String(""+((double)j/div));
			    g.drawString(s,30-offset,(int)pos);
			    g.drawString(s,30-offset,d.height/2+(int)pos);
			}
		    else
			{
			    s = new String(""+j/div);
			    g.drawString(s,32-offset,(int)pos);
			    g.drawString(s,32-offset,d.height/2+(int)pos);
			}
		    g.drawLine(28,(int)pos,30,(int)pos);
		    g.drawLine(28,(int)pos+d.height/2,30,(int)pos+d.height/2);
		    pos-=((double)(d.height/2-30)/(scale));
		}
	    if (showPoint != null)
		{
		    if (ss!=null)
			g.drawString("sent: "+ss, d.width-100, 10);
		    if (rs!=null)
			g.drawString("received: "+rs, d.width-100, 20);
		    if (rls!=null)
			g.drawString("rate limit: "+rls, d.width-100, 30);
		    g.setColor(Color.lightGray);
		    g.drawLine((int)showPoint.getX(),30,
			       (int)showPoint.getX(),d.height/2-10);
		    g.drawLine((int)showPoint.getX(),d.height/2+30,
			       (int)showPoint.getX(),d.height-10);
		}
	}
    } 

    /* Start the panel */
    public void start() {
	relaxer = new Thread(this);
	relaxer.start();
    }
}

  







