/*Copyright (c) 2003 The Regents of the University of California.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
     This product includes software developed by the University of
     California, Los Angeles and its contributors.
4. Neither the name of the University nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE. */

/* -*-  Mode:C++; c-basic-offset:2; tab-width:2; indent-tabs-mode:t -*- */

#ifndef COMMON_H
#define COMMON_H

#define ENTRY_SIZE 20
#define NUM_OF_TPARMS 8
#define MAX_REC_NUM 100
#define ENDNODE 1
#define BBNODE 2
#define END_DOMAIN -1

typedef enum _type
{   ADD_SENDER = 2, ADD_RECEIVER, ADD_ROUTER, ADD_BROUTER,
    ADD_DOMAIN, ADD_EDGE, ADD_TCP_SENDER, ADD_TCP_RECEIVER,
    REMOVE_SENDER, REMOVE_RECEIVER,
    REMOVE_ROUTER, REMOVE_EDGE, N_STEPS, QUIT, HANDOFF,
    QUIT_ACK, ADD_BIND, RESET = 100
}Type;

#define IROUTER 0
#define BROUTER 1

struct link_char {
  long bw;
  long delay;
  unsigned long dest;
  unsigned int domain;
  int quetype;
  int min_thres;
  int max_thres;
  int quesize;
  int pquesize;
};

typedef struct _request {
  Type type;
  int which;		// node id
  long source;		// changed to source for symmetry with dest
  int rec_num;	        // number of receivers a source sends to
  unsigned int dest[MAX_REC_NUM]; //array of receivers
  int domain;		// used only for routers and BBs
  int domaintype;       // used only for routers.
  int cost;		// cost
  long delay;		// one  way propagation delay
  long bw;		// link bw
  int quetype;	        // type of queue TAILDROP/RED/NRED
  int min_thres;	// minimum threshold for RED links
  int max_thres;	// maximum threshold for RED links
  int sender_type;	// Poisson/Pareto/Constant for UDP, FTP/TELNET for TCP
  int sender_traffic;   // BE/EF
  int data;		// amount of data sent in FTP con (in bytes)
  int sleep;		// time between FTP con (in ms)
  int start;		// start time for senders
  int quesize;	        // size of best effort queue (in pkts)
  int pquesize;	        // size of EF queue (in pkts)
  double tspecs[NUM_OF_TPARMS];// traffic chars (senders only)
  long address;
  int mask;
} Request;

typedef struct _domain{
  long IP_Prefix;	// domain's prefix
  int mask;		// mask
} Domain;

#define Reply Request
#define MAX_INTERFACE_NUM 24

int read_data(int, char *, int);
int write_data(int, char *, int);
void printIPAddress(long na, char*);
long parseIPAddress(char*);

typedef struct _Node {
  int type;		      // type of node, defined in messages.h
  unsigned int id;	      // The node's "address"
  unsigned int domain;	      // The node's domain (only for routers)
  int mask;		      // domain mask;
  long address;		      // node's IP address
  unsigned int domaintype;    // the type of domain(only for routers)
  struct link_char neighbors[MAX_INTERFACE_NUM];
  int howmany;	              // total number of neighbors
  long dest_sin_addr[MAX_REC_NUM];
  // needed for senders and receivers
  // and has different interpretation
  int num_of_rec;	      // Only for senders
  int sender_type;	      // Only for senders
  int sender_traffic;	      // Only for senders BE/EF
  double tspecs[NUM_OF_TPARMS];	// traffic parameters (snd only)
  int data;
  int sleep;
  int start;
} Node;


#endif // COMMON_H
