/*Copyright (c) 2003 The Regents of the University of California.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
     This product includes software developed by the University of
     California, Los Angeles and its contributors.
4. Neither the name of the University nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE. */

#ifndef __CHASH_H
#define __CHASH_H

#include "constants.h"
#include <linux/types.h>

/* Here we define type for connection key that 
   will be used for hashing */
struct conn_key
{
  unsigned int localIP;
  unsigned int foreignIP;
  unsigned short localport;
  unsigned short foreignport;
};

/* Connection record */
struct conn_stats{
  int protocol;
  int sentp;
  int sentB;
  int recp;
  int recB;
  int droppedp;
  int droppedB;
  int sum_sentp;
  int sum_sentB;
  int sum_recp;
  int sum_recB;
  int sum_droppedp;
  int sum_droppedB;
  int goodB;
  __u32 seqnum;
  __u32 acknum;
};

/* Functions defined in chash.c */
int conn_clean(struct conn_stats*);
int conn_compare(struct conn_key*, struct conn_key*);
unsigned long conn_hash1(struct conn_key*, unsigned long);
unsigned long conn_hash2(struct conn_key*, unsigned long);
void create_connhash();
int get_conn(unsigned int, unsigned int, unsigned short, unsigned short, int);
long insert_connection(unsigned int, unsigned int, unsigned short, unsigned short, int);
unsigned long find_connection(unsigned int, unsigned int, unsigned short, unsigned short);
void update_connection(unsigned int, unsigned int, unsigned short, 
		       unsigned short, int, enum direction,int,__u32,__u32);
int get_good(unsigned int, unsigned int, unsigned short, 
	      unsigned short);
void delete_connection(unsigned int, unsigned int, unsigned short, 
		       unsigned short);
void reset_conn(unsigned int, unsigned int, unsigned short, unsigned short);
void reset_conns();
void delete_connhash();
void update_conn_drop(unsigned int, unsigned int, unsigned short, unsigned short, int, int);

#endif
