/* Copyright (c) 2003 The Regents of the University of California.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
     This product includes software developed by the University of
     California, Los Angeles and its contributors.
4. Neither the name of the University nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE. */

/* System includes */
#include <linux/config.h> 
#include <linux/mm.h> 
#include <linux/module.h>
#include <linux/kernel.h> 
#include <asm/types.h> 
#include <linux/stddef.h> 
#include <linux/types.h> 
#include <linux/netfilter.h> 
#include <linux/netfilter_ipv4.h> 
#include <linux/skbuff.h> 
#include <linux/ip.h> 
#include <linux/tcp.h> 
#include <linux/icmp.h> 
#include <linux/in.h>
#include <linux/netdevice.h> 
#include <linux/if_ether.h> 
#include <sys/syscall.h>  
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/time.h>
#include <linux/types.h> 
#include <linux/netfilter.h> 
#include <linux/netfilter_ipv4.h> 
#include <linux/skbuff.h>
#include <linux/slab.h>

/* Our includes */
#include "khash.h"
#include "constants.h"
#include "chash.h"
#include "dward.config"

/* Table of good connections */
struct hash_table* ConnHash; 

/* Hash related functions we need to define */
int conn_clean(struct conn_stats* data)
{
  /* Never clean unless explicitly demanded */
    return 0;
}

int conn_compare(struct conn_key* k1, struct conn_key* k2)
{
  if (k1 == NULL || k2 == NULL)
    return 0;
  if (k1->localIP == k2->localIP &&
      k1->foreignIP == k2->foreignIP &&
      k1->localport == k2->localport &&
      k1->foreignport == k2->foreignport)
    return 1;
  else
    return 0;
}

unsigned long conn_hash1(struct conn_key* k, unsigned long size)
{
  /* Here we just hash sum of fields */
  return (k->localIP % size + k->foreignIP % size 
	  + k->localport + k->foreignport) % size;
}

unsigned long conn_hash2(struct conn_key* k, unsigned long size)
{
  /* Here we just hash sum of fields */
  return (1+conn_hash1(k,size-1));
}

void create_connhash()
{
  /* Create hash with same min and max size to avoid rehashing
     since we did not use locks for any operation */
  create_hash(&ConnHash, KERNEL_CONN_HASH_SIZE, 
	      KERNEL_CONN_HASH_SIZE,
	      KERNEL_CONN_HASH_SIZE, 
	      KERNEL_CONN_REHASH_LOAD_FACTOR_HIGH, 
	      KERNEL_CONN_REHASH_LOAD_FACTOR_LOW,  
	      sizeof(struct conn_key),
	      sizeof(struct conn_stats),
	      (int (*)(void*))conn_clean, 
	      (int (*)(void*,void*))conn_compare, 
	      (unsigned long (*)(void*,unsigned long))conn_hash1,
	      (unsigned long (*)(void*,unsigned long))conn_hash2);
}

/* Utility functions that mask hash structure */

/* Get connection statistics. This is a system call that 
   delivers more accurate statistics from kernel to application level */
int get_conn(unsigned int address1, unsigned int address2, 
		  unsigned short port1, unsigned short port2, int type)
{
  struct conn_key key;
  int index;
  key.localIP = address1;
  key.localport = port1;
  key.foreignIP = address2;
  key.foreignport = port2;
  index = find(&key, ConnHash);
  if (index != NOT_FOUND)
    {
      struct conn_stats* cs 
	= (struct conn_stats*)(ConnHash->entries[index].data);
      switch(type)
	{
	case PROTOCOL:
	  return cs->protocol;
	case SENT_PACKETS:
	  return cs->sum_sentp;
	case SENT_BYTES:
	  return cs->sum_sentB;
	case REC_PACKETS:
	  return cs->sum_recp;
	case REC_BYTES:
	  return cs->sum_recB;
	case DROP_PACKETS:
	  return cs->sum_droppedp;
	case DROP_BYTES:
	  return cs->sum_droppedB;
	default:
	  return 0;
	}
    }
  return 0;
}

/* Insert new good connection */
long insert_connection(unsigned int address1, unsigned int address2, 
		  unsigned short port1, unsigned short port2, int protocol)
{
  struct conn_key key;
  struct conn_stats* data; 
  int index, outcome;
  key.localIP = address1;
  key.localport = port1;
  key.foreignIP = address2;
  key.foreignport = port2;

  index = insert(&key, ConnHash,&outcome);
  if (index != NOT_FOUND && outcome == NEW)
    {
      data = (struct conn_stats*)ConnHash->entries[index].data;
      data->sum_sentp = 0;
      data->sum_sentB = 0;
      data->sum_recp = 0;
      data->sum_recB = 0;
      data->sum_droppedp = 0;
      data->sum_droppedB = 0;
      data->sentp = 0;
      data->sentB = 0;
      data->recp = 0;
      data->recB = 0;
      data->droppedp = 0;
      data->droppedB = 0;
      data->protocol = protocol;
      data->seqnum = 0;
      data->acknum = 0;
      data->goodB = 0;
      return index;
    }
  return NOT_FOUND;
}

/* Find conneciton and return its index in hash*/
unsigned long find_connection(unsigned int address1, unsigned int address2, 
		  unsigned short port1, unsigned short port2)
{
  struct conn_key key;
  key.localIP = address1;
  key.localport = port1;
  key.foreignIP = address2;
  key.foreignport = port2;
  return find(&key,ConnHash);
}

/* Update connection with new packet arrival */ 
void update_connection(unsigned int address1, unsigned int address2, 
		       unsigned short port1, unsigned short port2, int bytes, 
		       enum direction dir, int protocol, __u32 seqnum, 
		       __u32 acknum)
{
  struct conn_key key;
  int index;
  key.localIP = address1;
  key.foreignIP = address2;
  key.localport = port1;
  key.foreignport = port2;

  index = find(&key, ConnHash);
  if(index != NOT_FOUND)
    {
      	struct conn_stats* cs 
	  = (struct conn_stats*)(ConnHash->entries[index].data);
	
	/* Update all temporary values */
	if (dir == OUT)
	  {
	    cs->sentp ++;
	    cs->sentB += bytes;
	    /* Update sequence number - this is used for goodput calculation */
	    if (protocol == IPPROTO_TCP 
		&& ((unsigned long)seqnum > (unsigned long)cs->seqnum) 
		&& bytes > 52)
	      {
		cs->goodB += bytes;
		cs->seqnum = seqnum;
	      }
	  }
	else
	  {
	    cs->recp ++;
	    cs->recB += bytes;
	    if (protocol == IPPROTO_TCP 
		&& (unsigned long)acknum > (unsigned long)cs->acknum 
		&& bytes > 52)
	      {
		cs->goodB += bytes;
		cs->acknum = acknum;
	      }
	  }
    }
}

/* Update connection statistics on packet drop */
void update_conn_drop(unsigned int address1, unsigned int address2, 
		      unsigned short port1, unsigned short port2, int bytes, int gflag)
{
  struct conn_key key;
  int index;
  key.localIP = address1;
  key.localport = port1;
  key.foreignIP = address2;
  key.foreignport = port2;
  index = find(&key,ConnHash);
  if (index != NOT_FOUND)
    {
      	struct conn_stats* cs 
	  = (struct conn_stats*)(ConnHash->entries[index].data);
	/* Update temporary drop values */
	cs->droppedp ++;
	cs->droppedB += bytes;
    }
}

/* Get connection's goodput */
int get_good(unsigned int address1, unsigned int address2, 
	     unsigned short port1, unsigned short port2)
{
  struct conn_key key;
  int index, val=0;
  key.localIP = address1;
  key.foreignIP = address2;
  key.localport = port1;
  key.foreignport = port2;

  index = find(&key, ConnHash);
  if(index != NOT_FOUND)
    {
      struct conn_stats* cs 
	= (struct conn_stats*)(ConnHash->entries[index].data);
      
      /* Update all temporary values */
      val = cs->goodB;
      cs->goodB = 0;
    }
  return val;
}

/* Remove connection from hash */
void delete_connection(unsigned int address1, unsigned int address2, 
		  unsigned short port1, unsigned short port2)
{
  struct conn_key key;
  key.localIP = address1;
  key.localport = port1;
  key.foreignIP = address2;
  key.foreignport = port2;
  delete(&key,ConnHash);
}

/* Reset connection statistics */
void reset_conn(unsigned int address1, unsigned int address2, 
		  unsigned short port1, unsigned short port2)
{
  struct conn_key key;
  int index;
  key.localIP = address1;
  key.localport = port1;
  key.foreignIP = address2;
  key.foreignport = port2;
  index = find(&key,ConnHash);
  if (index != NOT_FOUND)
    {
      /* Reset all cumulative measures */
      struct conn_stats* cs 
	= (struct conn_stats*)(ConnHash->entries[index].data);
      cs->sum_sentp = 0;
      cs->sum_recp = 0;
      cs->sum_sentB = 0;
      cs->sum_recB = 0;
      cs->sum_droppedp = 0;
      cs->sum_droppedB = 0;
    }
}

/* Reset one-second statistics for all connections */
void reset_conns()
{
  int i;
  /* Add all one-second statistics onto cumulative statistics and 
     then reset one-second counters */
  for(i=0;i<ConnHash->size;i++)
    {
      if(ConnHash->entries[i].state == VALID)
	{
	  struct conn_stats* cs 
	  = (struct conn_stats*)(ConnHash->entries[i].data);
	  cs->sum_sentp += cs->sentp;
	  cs->sum_sentB += cs->sentB;
	  cs->sum_recp += cs->recp;
	  cs->sum_recB += cs->recB;
	  cs->sum_droppedp += cs->droppedp;
	  cs->sum_droppedB += cs->droppedB;
	  cs->sentp = 0;
	  cs->sentB = 0;
	  cs->recp = 0;
	  cs->recB = 0;
	  cs->droppedp = 0;
	  cs->droppedB = 0;
	}
    }
}

/* Delete hash table on exit */
void delete_connhash()
{
  int i;
  if (ConnHash == NULL || ConnHash->entries == NULL)
    return;
  for(i=0;i<ConnHash->size;i++)
    {
      if(ConnHash->entries[i].state == VALID)
	{
	  kfree(ConnHash->entries[i].key);	 
	  ConnHash->entries[i].key = NULL;
	  kfree(ConnHash->entries[i].data);
	  ConnHash->entries[i].data = NULL;
	}
    }
  kfree(ConnHash->entries);
  ConnHash->entries = NULL;
  kfree(ConnHash);
  ConnHash = NULL;
}



